@@ -8,7 +8,7 @@ Do not add this file to the repository.
 import datetime
 
 version = '2.1'
-date = 'Sun Mar 25 16:44:00 2018'
+date = 'Tue Aug  7 15:45:58 2018'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
@@ -18,7 +18,7 @@ dev = False
 version_info = ('networkx', '2', '1', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2018, 3, 25, 16, 44)
+date_info = datetime.datetime(2018, 8, 7, 15, 45, 58)
 
 # Format: (vcs, vcs_tuple)
 vcs_info = (None, (None, None))
